#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdbool.h>
#include <string.h>
#include <ctype.h>

#include "wordblox.h"
#include "constantstrings.h"

/* clear a cell in the grid */
void clear_cell (Puzzle *p)
{
	print_puzzle (p);
	int row, col;
	printf (INPUT_ROW);
	scanf ("%d", &row);
	printf (INPUT_COL);
	scanf ("%d", &col);
	if (row >= p->grid_size || col >= p->grid_size)
	{
		printf (EXCEED_GRID_SIZE);
		char ch = getchar ();
		ch = getchar ();
		return;
	}
	p->chars[row][col] = ' ';
	print_puzzle (p);
	
	char ch = getchar ();
	ch = getchar ();
}

/* add a down word to the grid */
void add_down_word (Puzzle *p)
{
	print_puzzle (p);
	if (p->grid_frozen) 
	{
		printf (FROZEN_GRID);
		char ch = getchar ();
		ch = getchar ();
		return;
	}
	char word[MAX_PUZZLE_SIZE];
	int row, col;
	printf (INPUT_WORD);
	scanf ("%s", word);
	if (! is_valid_word (word))
	{
		printf (INVALID_WORD);
		char ch = getchar ();
		ch = getchar ();
		return;
	}
	printf (INPUT_ROW);
	scanf ("%d", &row);
	printf (INPUT_COL);
	scanf ("%d", &col);
	if (row >= p->grid_size || col >= p->grid_size) 
	{
		printf (EXCEED_GRID_SIZE);
		char ch = getchar ();
		ch = getchar ();
		return;
	}
	if (strlen (word) > (p->grid_size - row))
	{
		printf (WORD_TOO_LONG);
		char ch = getchar ();
		ch = getchar ();
		return;
	}
	
	for (int i = row; i < row + strlen(word); i ++)
		p->chars[i][col] = toupper(word[i - row]);
	
	print_puzzle (p);
	char ch = getchar ();
	ch = getchar ();
}

/* add an across word to the grid */
void add_across_word (Puzzle *p)
{
	print_puzzle (p);
	if (p->grid_frozen) 
	{
		printf (FROZEN_GRID);
		return;
	}
	char word[MAX_PUZZLE_SIZE];
	int row, col;	
	printf (INPUT_WORD);
	scanf ("%s", word);
	if (! is_valid_word (word))
	{
		printf (INVALID_WORD);
		char ch = getchar ();
		ch = getchar ();
		return;
	}	
	printf (INPUT_ROW);
	scanf ("%d", &row);
	printf (INPUT_COL);
	scanf ("%d", &col);
	if (row >= p->grid_size || col >= p->grid_size)
	{
		printf (EXCEED_GRID_SIZE);
		char ch = getchar ();
		ch = getchar ();
		return;
	}
	
	if (strlen (word) > (p->grid_size - col))
	{
		printf (WORD_TOO_LONG);
		char ch = getchar ();
		ch = getchar ();
		return;
	}
	
	for (int i = col; i < col+strlen (word); i ++)
		p->chars[row][i] = toupper(word[i - col]);
		
	print_puzzle (p);
	char ch = getchar ();
	ch = getchar ();
}

void puzzle_editor_loop (Puzzle *p, const char *filename) 
{
	bool loop = true;
	while (loop) 
	{
		print_menu (WHITE, RED, PUZZLE_MENU_TITLE, PUZZLE_EDIT_MENU, 8, 50);
		printf (INPUT_CHOICE);
		char ch = getchar ();
		switch (ch)
		{
			case '1': print_puzzle (p);
					  char ch = getchar ();
					  ch = getchar ();
					  break;
			case '2': add_across_word (p);
					  break;
			case '3': add_down_word (p);
					  break;
			case '4': clear_cell (p);
					  break;
			case '7': save_puzzle (p, filename);
					  printf ("%s\n",FILE_SAVED);
					  ch = getchar ();
					  ch = getchar ();
					  break;
			case '8': loop = false;
					  break;
		}
	}
}

/* open an existing puzzle */
void open_puzzle () 
{
	Puzzle p;
	printf (INPUT_FILE);
	char filename[256];
	scanf("%s", filename);
	p = load_puzzle (filename);
	puzzle_editor_loop (&p, filename);
}

/* create a new blank puzzle */
void new_puzzle () 
{
	Puzzle p; 
	printf (INPUT_FILE);
	char filename[256]; 
	scanf ("%s", filename);
	printf (INPUT_GRID_SIZE);
	int size;
	scanf ("%d", &size);
	if (size > MAX_PUZZLE_SIZE) 
	{ 
		printf (EXCEED_MAX_GRID_SIZE);
		char c = getchar (); c = getchar ();
	}
	else
	{
		init_puzzle (&p, size);
		puzzle_editor_loop (&p, filename);
	}
}

/* The main loop of the program */
int main_loop () 
{
	/* Print the main menu */
	while (1)
	{
		print_menu (WHITE, BLUE, MAIN_MENU_TITLE, MAIN_MENU, 3, 50);
		printf (INPUT_CHOICE);
		char ch = getchar ();
		switch (ch)
		{
			case '1': new_puzzle ();
					  break;
			case '2': open_puzzle ();
					  break;
			case '3': exit (0);
		}
	}
}

int main (int argc, char* argv[]) 
{	
	return (main_loop ());
}
